from setuptools import setup

setup(
    name='anyrpc',
    version='0.0.1',
    author='fuchen',
    author_email='ink.pad@163.com',
    url='https://github.com/fuchen/any-rpc-py',
    description=u'Python implementation of any-rpc',
    packages=['anyrpc'],
    install_requires=[],
    entry_points={
    }
)
